#include <cmath>
#include "Sphere.h"

#include "collide.h"

void collide(Sphere &a, Sphere &b, const double cor)
    {

    // We need a variable to store a vectors length
    double length = 0;

    // We need variables for the two helper vectors
    double hx = 0;
    double hy = 0;
    double hz = 0;

    // Calculate the orthogonal component from the viewpoint of the center of sphere a

        // Calculate the direction first
        double ueao_x = b.px - a.px;
        double ueao_y = b.py - a.py;
        double ueao_z = b.pz - a.pz;

        // Make ueao a unit vector
        length = std::sqrt( std::pow(ueao_x, 2)+std::pow(ueao_y, 2)+std::pow(ueao_z, 2) );
        ueao_x /= length;
        ueao_y /= length;
        ueao_z /= length;

    // Calculate the parallel component of sphere a

        // First vector product (helper vector)
        hx = a.vy * ueao_z - a.vz * ueao_y;
        hy = a.vz * ueao_x - a.vx * ueao_z;
        hz = a.vx * ueao_y - a.vy * ueao_x;

        // Declare the parallel vector for b
        double ueap_x = 0;
        double ueap_y = 0;
        double ueap_z = 0;

        if ( hx == 0 && hy == 0 && hz == 0 )
            {
            ueap_x = 0;
            ueap_y = 0;
            ueap_z = 0;
            }
        else
            {
            // 2nd vector product to get the parallel vector
            ueap_x = hy * ueao_z - hz * ueao_y;
            ueap_y = hz * ueao_x - hx * ueao_z;
            ueap_z = hx * ueao_y - hy * ueao_x;

            // Make ueap a unit vector
            length = std::sqrt( std::pow(ueap_x, 2)+std::pow(ueap_y, 2)+std::pow(ueap_z, 2) );
            ueap_x /= length;
            ueap_y /= length;
            ueap_z /= length;
            }

    // Calculate the parallel component of sphere b

        // First the helper vector
        hx = b.vy * ueao_z - b.vz * ueao_y;
        hy = b.vz * ueao_x - b.vx * ueao_z;
        hz = b.vx * ueao_y - b.vy * ueao_x;

        // Declare the parallel vector for b
        double uebp_x = 0;
        double uebp_y = 0;
        double uebp_z = 0;

        if ( hx == 0 && hy == 0 && hz == 0 )
            {
            uebp_x = 0;
            uebp_y = 0;
            uebp_z = 0;
            }
        else
            {
            // Parallel vector
            uebp_x = hy * ueao_z - hz * ueao_y;
            uebp_y = hz * ueao_x - hx * ueao_z;
            uebp_z = hx * ueao_y - hy * ueao_x;

            // Make uebp a unit vector
            length = std::sqrt( std::pow(uebp_x, 2)+std::pow(uebp_y, 2)+std::pow(uebp_z, 2) );
            uebp_x /= length;
            uebp_y /= length;
            uebp_z /= length;
            }

    // Calculate the scalars lao, lap, lbo and lbp
    const double lao = ueao_x * a.vx + ueao_y * a.vy + ueao_z * a.vz;
    const double lap = ueap_x * a.vx + ueap_y * a.vy + ueap_z * a.vz;
    const double lbo = ueao_x * b.vx + ueao_y * b.vy + ueao_z * b.vz;
    const double lbp = uebp_x * b.vx + uebp_y * b.vy + uebp_z * b.vz;

    // For faster Code we will calculate the long terms in the formula once while we need them three times
    const double factor_a = ( lao * a.mass + lbo * b.mass + (lbo - lao) * b.mass * cor ) / ( a.mass + b.mass );
    const double factor_b = ( lao * a.mass + lbo * b.mass + (lao - lbo) * a.mass * cor ) / ( a.mass + b.mass );

    // Calculate the resulting velocity vectors
    a.vx = factor_a * ueao_x + lap * ueap_x;
    a.vy = factor_a * ueao_y + lap * ueap_y;
    a.vz = factor_a * ueao_z + lap * ueap_z;

    b.vx = factor_b * ueao_x + lbp * uebp_x;
    b.vy = factor_b * ueao_y + lbp * uebp_y;
    b.vz = factor_b * ueao_z + lbp * uebp_z;

    return;
    }